Package: shiny.tailwind
Type: Package
Title: 'TailwindCSS' for Shiny Apps
Version: 0.2.2
Authors@R: c(
      person(given = "Kyle",
             family = "Butts",
             role = c("aut","cre"),
             email = "kyle.butts@colorado.edu",
             comment = c(ORCID = "0000-0002-9048-8059")),
      person(given = "David",
             family = "Zimmermann-Kollenda",
             role = "ctb",
             email = "david_j_zimmermann@hotmail.com"))
Description: Allows 'TailwindCSS' to be used in Shiny apps with just-in-time 
  compiling, custom css with '@apply' directive, and custom tailwind 
  configurations.
Encoding: UTF-8
RoxygenNote: 7.2.1
License: MIT + file LICENSE
Depends: R (>= 3.6.0), htmltools
Imports: shiny
URL: https://github.com/kylebutts/shiny.tailwind
BugReports: https://github.com/kylebutts/shiny.tailwind/issues
Suggests: fontawesome, palmerpenguins, ggplot2
NeedsCompilation: no
Packaged: 2022-10-11 20:30:49 UTC; kylebutts
Author: Kyle Butts [aut, cre] (<https://orcid.org/0000-0002-9048-8059>),
  David Zimmermann-Kollenda [ctb]
Maintainer: Kyle Butts <kyle.butts@colorado.edu>
Repository: CRAN
Date/Publication: 2022-10-13 08:50:02 UTC
Built: R 4.6.0; ; 2026-04-28 21:14:15 UTC; unix
